@@ -38,6 +38,10 @@ def test_connection_pool_with_keepalive():
             assert info == [
                 "<HTTPConnection ['https://example.com:443', HTTP/1.1, ACTIVE, Request Count: 1]>"
             ]
+            assert (
+                repr(pool)
+                == "<ConnectionPool [Requests: 1 active, 0 queued | Connections: 1 active, 0 idle]>"
+            )
             response.read()
 
         assert response.status == 200
@@ -46,6 +50,10 @@ def test_connection_pool_with_keepalive():
         assert info == [
             "<HTTPConnection ['https://example.com:443', HTTP/1.1, IDLE, Request Count: 1]>"
         ]
+        assert (
+            repr(pool)
+            == "<ConnectionPool [Requests: 0 active, 0 queued | Connections: 0 active, 1 idle]>"
+        )
 
         # Sending a second request to the same origin will reuse the existing IDLE connection.
         with pool.stream("GET", "https://example.com/") as response:
@@ -53,6 +61,10 @@ def test_connection_pool_with_keepalive():
             assert info == [
                 "<HTTPConnection ['https://example.com:443', HTTP/1.1, ACTIVE, Request Count: 2]>"
             ]
+            assert (
+                repr(pool)
+                == "<ConnectionPool [Requests: 1 active, 0 queued | Connections: 1 active, 0 idle]>"
+            )
             response.read()
 
         assert response.status == 200
@@ -61,23 +73,35 @@ def test_connection_pool_with_keepalive():
         assert info == [
             "<HTTPConnection ['https://example.com:443', HTTP/1.1, IDLE, Request Count: 2]>"
         ]
+        assert (
+            repr(pool)
+            == "<ConnectionPool [Requests: 0 active, 0 queued | Connections: 0 active, 1 idle]>"
+        )
 
         # Sending a request to a different origin will not reuse the existing IDLE connection.
         with pool.stream("GET", "http://example.com/") as response:
             info = [repr(c) for c in pool.connections]
             assert info == [
-                "<HTTPConnection ['http://example.com:80', HTTP/1.1, ACTIVE, Request Count: 1]>",
                 "<HTTPConnection ['https://example.com:443', HTTP/1.1, IDLE, Request Count: 2]>",
+                "<HTTPConnection ['http://example.com:80', HTTP/1.1, ACTIVE, Request Count: 1]>",
             ]
+            assert (
+                repr(pool)
+                == "<ConnectionPool [Requests: 1 active, 0 queued | Connections: 1 active, 1 idle]>"
+            )
             response.read()
 
         assert response.status == 200
         assert response.content == b"Hello, world!"
         info = [repr(c) for c in pool.connections]
         assert info == [
-            "<HTTPConnection ['http://example.com:80', HTTP/1.1, IDLE, Request Count: 1]>",
             "<HTTPConnection ['https://example.com:443', HTTP/1.1, IDLE, Request Count: 2]>",
+            "<HTTPConnection ['http://example.com:80', HTTP/1.1, IDLE, Request Count: 1]>",
         ]
+        assert (
+            repr(pool)
+            == "<ConnectionPool [Requests: 0 active, 0 queued | Connections: 0 active, 2 idle]>"
+        )
 
 
 
@@ -219,6 +243,7 @@ def test_connection_pool_with_http2_goaway():
         ]
 
         # Sending a second request to the same origin will require a new connection.
+        # The original connection has now been closed.
         response = pool.request("GET", "https://example.com/")
         assert response.status == 200
         assert response.content == b"Hello, world!"
@@ -226,7 +251,6 @@ def test_connection_pool_with_http2_goaway():
         info = [repr(c) for c in pool.connections]
         assert info == [
             "<HTTPConnection ['https://example.com:443', HTTP/2, IDLE, Request Count: 1]>",
-            "<HTTPConnection ['https://example.com:443', HTTP/2, CLOSED, Request Count: 1]>",
         ]
 
 
@@ -620,6 +644,11 @@ def test_connection_pool_concurrency_same_domain_keepalive():
                 "<HTTPConnection ['https://a.com:443', HTTP/1.1, ACTIVE, Request Count: 5]>",
             ]
 
+    assert (
+        repr(pool)
+        == "<ConnectionPool [Requests: 0 active, 0 queued | Connections: 0 active, 0 idle]>"
+    )
+
 
 
 def test_unsupported_protocol():
@@ -767,6 +796,12 @@ def test_http11_upgrade_connection():
             b"...",
         ]
     )
+
+    called = []
+
+    def trace(name, kwargs):
+        called.append(name)
+
     with httpcore.ConnectionPool(
         network_backend=network_backend, max_connections=1
     ) as pool:
@@ -774,8 +809,24 @@ def test_http11_upgrade_connection():
             "GET",
             "wss://example.com/",
             headers={"Connection": "upgrade", "Upgrade": "custom"},
+            extensions={"trace": trace},
         ) as response:
             assert response.status == 101
             network_stream = response.extensions["network_stream"]
             content = network_stream.read(max_bytes=1024)
             assert content == b"..."
+
+    assert called == [
+        "connection.connect_tcp.started",
+        "connection.connect_tcp.complete",
+        "connection.start_tls.started",
+        "connection.start_tls.complete",
+        "http11.send_request_headers.started",
+        "http11.send_request_headers.complete",
+        "http11.send_request_body.started",
+        "http11.send_request_body.complete",
+        "http11.receive_response_headers.started",
+        "http11.receive_response_headers.complete",
+        "http11.response_closed.started",
+        "http11.response_closed.complete",
+    ]
